from enigma import eServiceCenter, eServiceReference, pNavigation, getBestPlayableServiceReference, iPlayableService
from Components.config import config
from Components.ParentalControl import parentalControl
from Components.config import config
from Tools.BoundFunction import boundFunction
from Tools.DreamboxHardware import getFPWasTimerWakeup, clearFPWasTimerWakeup
from Tools import Notifications
import RecordTimer
import SleepTimer
import Screens.Standby
import NavigationInstance
import ServiceReference

# TODO: remove pNavgation, eNavigation and rewrite this stuff in python.
class Navigation:
	def __init__(self, nextRecordTimerAfterEventActionAuto=False):
		from Screens.ChannelSelection import ChannelSelection
		if NavigationInstance.instance is not None:
			raise NavigationInstance.instance
		
		NavigationInstance.instance = self
		self.ServiceHandler = eServiceCenter.getInstance()
		
		import Navigation as Nav
		Nav.navcore = self
		
		self.pnav = pNavigation()
		self.nav_event_conn = self.pnav.m_event.connect(self.dispatchEvent)
		self.nav_record_event_conn = self.pnav.m_record_event.connect(self.dispatchRecordEvent)
		self.event = [ ]
		self.record_event = [ ]
		self.currentlyPlayingServiceReference = None
		self.currentlyPlayingService = None
		self.immediatePlay = False
		self.RecordTimer = RecordTimer.RecordTimer()
		self.__wasTimerWakeup = False
		if getFPWasTimerWakeup():
			clearFPWasTimerWakeup()
			if getFPWasTimerWakeup(): # sanity check to detect if the FP driver is working correct!
				print "buggy fp driver detected!!! please update drivers.... ignore timer wakeup!"
			else:
				self.__wasTimerWakeup = True
				if config.misc.prev_wakeup_time.value and config.misc.prev_wakeup_time_type.value == 0:
					if not Screens.Standby.inTryQuitMainloop: # not a shutdown messagebox is open
						Notifications.AddNotificationWithID("Standby", Screens.Standby.Standby, domain="RecordTimer")
		self.SleepTimer = SleepTimer.SleepTimer()
		self.scheduledServiceReference = None
		ChannelSelection.initialized.addNotifier(self.serviceListInitialized, initial_call = True)

	def serviceListInitialized(self, configElement):
		self.immediatePlay = configElement.value
		if self.immediatePlay and self.scheduledServiceReference is not None:
			print "deferred playService ... ChannelSelection is finally initialized now"
			ref = self.scheduledServiceReference
			self.scheduledServiceReference = None
			self.playService(ref[0], ref[1], ref[2])

	def wasTimerWakeup(self):
		return self.__wasTimerWakeup

	def dispatchEvent(self, i):
		for x in self.event:
			x(i)
		if i == iPlayableService.evEnd:
			self.currentlyPlayingServiceReference = None
			self.currentlyPlayingService = None

	def dispatchRecordEvent(self, rec_service, event):
#		print "record_event", rec_service, event
		for x in self.record_event:
			x(rec_service, event)

	def playService(self, ref, checkParentalControl = True, forceRestart = False):
		if not self.immediatePlay:
			print "delaying playService request until the ChannelSelection is finally initialized"
			self.scheduledServiceReference = (ref, checkParentalControl, forceRestart)
			return
		oldref = self.currentlyPlayingServiceReference
		if ref and oldref and ref == oldref and not forceRestart:
			print "ignore request to play already running service(1)"
			return 0
		print "playing", ref and ref.toString()
		if ref is None:
			self.stopService()
			return 0
		if not checkParentalControl or parentalControl.isServicePlayable(ref, boundFunction(self.playService, checkParentalControl = False)):
			if ref.flags & eServiceReference.isGroup:
				if not oldref:
					oldref = eServiceReference()
				playref = getBestPlayableServiceReference(ref, oldref)
				print "playref", playref
				if playref and oldref and playref == oldref and not forceRestart:
					print "ignore request to play already running service(2)"
					return 0
				if not playref or (checkParentalControl and not parentalControl.isServicePlayable(playref, boundFunction(self.playService, checkParentalControl = False))):
					self.stopService()
					return 0
			else:
				playref = ref
			if self.pnav and not self.pnav.playService(playref):
				self.currentlyPlayingServiceReference = self.pnav.getCurrentServiceReference();
				return 0
		else:
			self.stopService()
		return 1

	# You'll break ServiceEventTracker if you ever change this, keep your hand's of it!
	# It HAS to be set after evStart
	def getCurrentlyPlayingServiceReference(self):
		return self.currentlyPlayingServiceReference

	def getCurrentServiceReference(self):
		return self.pnav.getCurrentServiceReference()

	def recordService(self, ref, simulate=False):
		service = None
		if not simulate:
			print "recording service: %s" % (str(ref))
		if isinstance(ref, ServiceReference.ServiceReference):
			ref = ref.ref
		if ref:
			if ref.flags & eServiceReference.isGroup:
				ref = getBestPlayableServiceReference(ref, eServiceReference(), simulate)
			service = ref and self.pnav and self.pnav.recordService(ref, simulate)
			if service is None:
				if simulate:
					print "simulate recording service: %s failed" % (str(ref))
				print "record returned non-zero"
		return service

	def stopRecordService(self, service):
		ret = self.pnav and self.pnav.stopRecordService(service)
		return ret

	def getRecordings(self, simulate=False):
		return self.pnav and self.pnav.getRecordings(simulate)

	def getCurrentService(self):
		if not self.currentlyPlayingService:
			self.currentlyPlayingService = self.pnav and self.pnav.getCurrentService()
		return self.currentlyPlayingService

	def stopService(self):
		if not self.immediatePlay:
			self.scheduledServiceReference = None
			return
		print "stopService"
		if self.pnav:
			self.pnav.stopService()

	def pause(self, p):
		return self.pnav and self.pnav.pause(p)

	def shutdown(self):
		self.RecordTimer.shutdown()
		self.ServiceHandler = None
		self.pnav = None

	def stopUserServices(self):
		self.stopService()
